﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace LoadScp
{
   public partial class LoadScp : Form
   {
      public LoadScp()
      {
         InitializeComponent();
      }

      private static VisaInstrument mScope;
      private const string mStrBtnReady = "Find and Load .scp File";
      private const string mStrBtnLoading = "Loading ...";

      private void mBtnSend_Click(object sender, EventArgs e)
      {
         try
         {
            mScope = new VisaInstrument(mTxtAddress.Text);
            mScope.SetTimeoutSeconds(60);

            OpenFileDialog selectFileDialog = new OpenFileDialog();
            selectFileDialog.Filter = "(*.scp)|*.scp";

            if (selectFileDialog.ShowDialog() == DialogResult.OK)
            {
               mBtnSend.Text = mStrBtnLoading;
               mBtnSend.Enabled = false;

               byte[] DataArray;
               DataArray = File.ReadAllBytes(selectFileDialog.FileName);
               mScope.WriteIEEEBlock(":SYSTem:SETup", DataArray);
               MessageBox.Show("Sent SCP file to scope.");

               mBtnSend.Text = mStrBtnReady;
               mBtnSend.Enabled = true;
            }
         }
         catch (Exception ex)
         {
            MessageBox.Show(ex.Message);
         }
      }
   }

   class VisaInstrument
   {
      private int m_nResourceManager;
      private int m_nSession;
      private string m_strVisaAddress;

      public VisaInstrument(string strVisaAddress)
      {
         // Save VISA addres in member variable.
         m_strVisaAddress = strVisaAddress;

         // Open the default VISA resource manager.
         OpenResourceManager();

         // Open a VISA resource session.
         OpenSession();

         // Clear the interface.
         int nViStatus;
         nViStatus = visa32.viClear(m_nSession);
      }

      public void Write(string strCommand)
      {
         // Send the command.
         VisaSendCommandOrQuery(strCommand);

         // Check for inst errors.
         CheckInstrumentErrors(strCommand);
      }

      public int WriteIEEEBlock(string strCommand, byte[] DataArray)
      {
         // Send the command to the device.
         string strCommandAndLength;
         int nViStatus, nLength, nBytesWritten;

         nLength = DataArray.Length;
         strCommandAndLength = String.Format("{0} #8{1:00000000}", strCommand, nLength);

         // Write first part of command to formatted I/O write buffer.
         nViStatus = visa32.viPrintf(m_nSession, strCommandAndLength);
         CheckVisaStatus(nViStatus);

         // Write the data to the formatted I/O write buffer.
         nViStatus = visa32.viBufWrite(m_nSession, DataArray, nLength, out nBytesWritten);
         CheckVisaStatus(nViStatus);

         // Check for inst errors.
         CheckInstrumentErrors(strCommand);

         return nBytesWritten;
      }

      public StringBuilder ReadString(string strQuery)
      {
         // Send the query.
         VisaSendCommandOrQuery(strQuery);

         // Get the result string.
         StringBuilder strResults = new StringBuilder(1000);
         strResults = VisaGetResultString();

         // Check for inst errors.
         CheckInstrumentErrors(strQuery);

         // Return string results.
         return strResults;
      }

      public double ReadDbl(string strQuery)
      {
         // Send the query.
         VisaSendCommandOrQuery(strQuery);

         // Get the result string.
         double fResults;
         fResults = VisaGetResultNumber();

         // Check for inst errors.
         CheckInstrumentErrors(strQuery);

         // Return string results.
         return fResults;
      }

      public double[] ReadDblList(string strQuery)
      {
         // Send the query.
         VisaSendCommandOrQuery(strQuery);

         // Get the result string.
         double[] fResultsArray;
         fResultsArray = VisaGetResultNumbers();

         // Check for inst errors.
         CheckInstrumentErrors(strQuery);

         // Return string results.
         return fResultsArray;
      }

      public int ReadIEEEBlock(string strQuery, out byte[] ResultsArray)
      {
         // Send the query.
         VisaSendCommandOrQuery(strQuery);

         // Get the result string.
         int length;   // Number of bytes returned from instrument.
         length = VisaGetResultIEEEBlock(out ResultsArray);

         // Check for inst errors.
         CheckInstrumentErrors(strQuery);

         // Return string results.
         return length;
      }

      private void VisaSendCommandOrQuery(string strCommandOrQuery)
      {
         // Send command or query to the device.
         string strWithNewline;
         strWithNewline = String.Format("{0}\n", strCommandOrQuery);
         int nViStatus;
         nViStatus = visa32.viPrintf(m_nSession, strWithNewline);
         CheckVisaStatus(nViStatus);
      }

      private StringBuilder VisaGetResultString()
      {
         StringBuilder strResults = new StringBuilder(1000);

         // Read return value string from the device.
         int nViStatus;
         nViStatus = visa32.viScanf(m_nSession, "%1000t", strResults);
         CheckVisaStatus(nViStatus);

         return strResults;
      }

      private double VisaGetResultNumber()
      {
         double fResults = 0;

         // Read return value string from the device.
         int nViStatus;
         nViStatus = visa32.viScanf(m_nSession, "%lf", out fResults);
         CheckVisaStatus(nViStatus);

         return fResults;
      }

      private double[] VisaGetResultNumbers()
      {
         double[] fResultsArray;
         fResultsArray = new double[10];

         // Read return value string from the device.
         int nViStatus;
         nViStatus = visa32.viScanf(m_nSession, "%,10lf\n",
             fResultsArray);
         CheckVisaStatus(nViStatus);

         return fResultsArray;
      }

      private int VisaGetResultIEEEBlock(out byte[] ResultsArray)
      {
         // Results array, big enough to hold a PNG.
         ResultsArray = new byte[300000];
         int length;   // Number of bytes returned from instrument.

         // Set the default number of bytes that will be contained in
         // the ResultsArray to 300,000 (300kB).
         length = 300000;

         // Read return value string from the device.
         int nViStatus;
         nViStatus = visa32.viScanf(m_nSession, "%#b", ref length,
           ResultsArray);
         CheckVisaStatus(nViStatus);

         // Write and read buffers need to be flushed after IEEE block?
         nViStatus = visa32.viFlush(m_nSession, visa32.VI_WRITE_BUF);
         CheckVisaStatus(nViStatus);

         nViStatus = visa32.viFlush(m_nSession, visa32.VI_READ_BUF);
         CheckVisaStatus(nViStatus);

         return length;
      }

      private void CheckInstrumentErrors(string strCommand)
      {
         // Check for instrument errors.
         StringBuilder strInstrumentError = new StringBuilder(1000);
         bool isFirstError = true;

         do   // While not "0,No error"
         {
            VisaSendCommandOrQuery(":SYSTem:ERRor?");
            strInstrumentError = VisaGetResultString();

            if (!strInstrumentError.ToString().StartsWith("+0,"))
            {
               if (isFirstError)
               {
                  Console.WriteLine("ERROR(s) for command '{0}': ", strCommand);
                  isFirstError = false;
               }

               Console.Write(strInstrumentError);
            }
         } 
         while (!strInstrumentError.ToString().StartsWith("+0,"));
      }

      private void OpenResourceManager()
      {
         int nViStatus;
         nViStatus = visa32.viOpenDefaultRM(out this.m_nResourceManager);
         if (nViStatus < visa32.VI_SUCCESS)
         {
            throw new ApplicationException("Failed to open Resource Manager");
         }
      }

      private void OpenSession()
      {
         int nViStatus;
         nViStatus = visa32.viOpen(this.m_nResourceManager, 
                                   this.m_strVisaAddress, 
                                   visa32.VI_NO_LOCK,
                                   visa32.VI_TMO_IMMEDIATE, 
                                   out this.m_nSession);
         CheckVisaStatus(nViStatus);
      }

      public void SetTimeoutSeconds(int nSeconds)
      {
         int nViStatus;
         nViStatus = visa32.viSetAttribute(this.m_nSession,
                                           visa32.VI_ATTR_TMO_VALUE, 
                                           nSeconds * 1000);
         CheckVisaStatus(nViStatus);
      }

      public void CheckVisaStatus(int nViStatus)
      {
         // If VISA error, throw exception.
         if (nViStatus < visa32.VI_SUCCESS)
         {
            StringBuilder strError = new StringBuilder(256);
            visa32.viStatusDesc(this.m_nResourceManager, nViStatus, strError);
            throw new ApplicationException(strError.ToString());
         }
      }

      public void Close()
      {
         if (m_nSession != 0)
         {
            visa32.viClose(m_nSession);
         }

         if (m_nResourceManager != 0)
         {
            visa32.viClose(m_nResourceManager);
         }
      }
   }
}
